<?php

namespace Pinpoint\CarouselManager\Block;

use Magento\Framework\View\Element\Template;
/**
 * Class Carousel
 * @package Pinpoint\CarouselManager\Block
 */
class Carousel extends Template implements \Magento\Widget\Block\BlockInterface
{
    /**
     * @var
     */
    protected $carouselId;
    /**
     * @var \Pinpoint\CarouselManager\Model\CarouselRepository
     */
    protected $carouselRespository;
    /**
     * @var
     */
    protected $carousel;

    /**
     * @var string
     */
    protected $_template;

    /**
     * @var string
     */
    protected $templateFilterModel = 'Magento\Catalog\Model\Template\Filter';
    /**
     * @var
     */
    protected $templateProcessor;
    /**
     * @var \Magento\Catalog\Model\Template\Filter\Factory
     */
    protected $templateFilterFactory;

    /**
     * @var array
     */
    protected $slides = [];

    /**
     * Carousel constructor.
     * @param \Pinpoint\CarouselManager\Model\CarouselRepository $carouselRepository
     * @param \Magento\Catalog\Model\Template\Filter\Factory $templateFilterFactory
     * @param Template\Context $context
     * @param array $data
     */
    public function __construct(
        \Pinpoint\CarouselManager\Model\CarouselRepository $carouselRepository,
        \Magento\Catalog\Model\Template\Filter\Factory $templateFilterFactory,
        Template\Context $context,
        array $data = []
    ) {
        $this->templateFilterFactory = $templateFilterFactory;
        $this->carouselRespository = $carouselRepository;

        $this->_template = 'Pinpoint_CarouselManager::carousel.phtml';

        if (isset($data['template']) && $data['template']) {
            $this->setTemplate($data['template']);
        }

        if (isset($data['carousel_id']) && $data['carousel_id']) {
            $this->setCarouselId($data['carousel_id']);
        }

        parent::__construct($context, $data);
    }

    /**
     * @return mixed
     */
    public function getCarouselId()
    {
        return $this->carouselId;
    }

    /**
     * @param $carouselId
     */
    public function setCarouselId($carouselId)
    {
        $this->carouselId = $carouselId;
    }

    /**
     * @return mixed
     */
    public function getCurrentStore()
    {
        return $this->_storeManager->getStore()->getId();
    }

    /**
     * @return mixed|null
     */
    public function getCarousel()
    {
        if (!$this->carousel) {

            try {

                $carouselId = $this->getCarouselId();
                $carousel = $this->carouselRespository->getById($carouselId);
                $currentStoreId = $this->getCurrentStore();

            } catch (\Exception $e) {
                // Silently skip
                $this->_logger->critical($e);
                return null;
            }

            $allowStores = $carousel->getStoreIds();

            if ($allowStores && $currentStoreId && is_array($allowStores) && count($allowStores) && in_array($currentStoreId, $allowStores)) {
                $this->carousel = $carousel;
            }

        }

        return $this->carousel;
    }

    /**
     * @return string
     */
    protected function _toHtml()
    {
        if (!$this->getCarousel()) {
            return '';
        }
        return parent::_toHtml();
    }

    /**
     * @return mixed
     */
    protected function _getTemplateProcessor()
    {
        if (null === $this->templateProcessor) {
            $this->templateProcessor = $this->templateFilterFactory->create($this->templateFilterModel);
        }

        return $this->templateProcessor;
    }

    /**
     * @param $attribute
     * @return bool
     */
    public function isFilterable($attribute)
    {
        return in_array($attribute, ['title', 'description', 'cta', 'link']);
    }

    /**
     * @param $slide
     * @param $attribute
     * @param string $default
     * @return mixed
     */
    public function getSlideProp($slide, $attribute, $default = '')
    {
        if (!isset($this->slides[$slide->getId()][$attribute])) {
            $val = $slide->getData($attribute) ? $slide->getData($attribute) : $default;
            if ($this->isFilterable($attribute)) {
                $val = $this->_getTemplateProcessor()->filter($val);
            }

            $this->slides[$slide->getId()][$attribute] = $val;
        }
        return $this->slides[$slide->getId()][$attribute];
    }

    /**
     * @param $slide
     * @return mixed
     */
    public function getImageUrl($slide)
    {
        if (!isset($this->slides[$slide->getId()]['image'])) {
            $this->slides[$slide->getId()]['image'] = null;
            $imagePath = $this->getSlideProp($slide, 'image', null);
            if ($imagePath) {
                $imageUrl = $this->getBaseUrl() . $imagePath;
                if ($imageUrl) {
                    $this->slides[$slide->getId()]['image'] = $imageUrl;
                }
            }

        }
        return $this->slides[$slide->getId()]['image'];
    }


}